/***********************************************************
constants
***********************************************************/

// reference to the interface defined in cmguiIGeneralThresholdFilter.idl
const cmguiIGeneralThresholdFilter = Components.interfaces.cmguiIGeneralThresholdFilter;

// reference to the required base interface that all components must support
const nsISupports = Components.interfaces.nsISupports;

// references to other required interfaces
// const nsIDOMWindow = Components.interfaces.nsIDOMWindow;

// UUID uniquely identifying our component
// You can get from: http://kruithof.xs4all.nl/uuid/uuidgen here
const CLASS_ID = Components.ID("{542E27CC-9F80-11DB-B3B1-507056D89593}");

// description
const CLASS_NAME = "Cmgui general threshold filter XPCOM Component";

// textual unique identifier
const CONTRACT_ID = "@bioeng.auckland.ac.nz/generalthresholdfilter;1";

/***********************************************************
class definition
***********************************************************/

//class constructor
function GeneralThresholdFilter() {
};


// class definition
GeneralThresholdFilter.prototype = {

  // members
  mode: 0,  // 0 is below, 1 is above, 2 is outside	
  outsideValue: 0.5,
  belowValue: 0.5,
  aboveValue: 0.5,
  inputField: null,
  outputField: null,
	callback: null,
	resampledInputFieldName: null,
	previewOutputFieldName: null,
	resampledInputField: null,
	previewOutputField: null,
	gElementName: null,
	sceneViewer: null,

// define the function methods we want to expose in our interface

  init: function(parentWindow, commandData, inputFieldName, outputFieldName, callback) {

		this.parent = parentWindow;
		this.commandData = commandData;
		this.inputFieldName = inputFieldName;
		this.outputFieldName = outputFieldName;
		this.callback = callback;

		// find the input field, note it is assumed to exist
	  this.inputField = commandData.findFieldByName(inputFieldName);

		// set some defaults which will be overwritten if the field exists
		this.mode = 0;  // 0 corresponds to below mode
		this.outsideValue = 1.0;
		this.belowValue = 0.5;
		this.aboveValue = 0.5;


		// check if output field exists
		if( commandData.isFieldDefined(outputFieldName) ){

			this.outputField = commandData.findFieldByName(outputFieldName);

			// create some objects which will be populated by the current field
			// properties if the field is the right type
			
			var returnInputField = new Object();
			var belowValue = new Object();
			var aboveValue = new Object();
			var outsideValue = new Object();
			var mode = new Object();					

			try {

				this.outputField.getTypeThresholdImageFilter(returnInputField,mode,outsideValue,belowValue,aboveValue);
				
				// when fetched the parameters are assigned to the value property 
				this.mode = mode.value;
				this.outsideValue = outsideValue.value;
				this.belowValue = belowValue.value;
				this.aboveValue = aboveValue.value;

			} catch(ex) {

				this.parent.alert("Error fetching output field parameters.  The field exists but is probably of the wrong type:" + ex);

				this.outputField.setTypeThresholdImageFilter(this.inputField,this.belowValue,this.aboveValue,this.outsideValue,this.mode);

			}
			
		} else {

			// field does not exist so create it.

			this.outputField = commandData.createField(outputFieldName);
			this.outputField.setTypeThresholdImageFilter(this.inputField,this.mode,this.outsideValue,
																									 this.belowValue,this.aboveValue);
		}
    
  },


	setup_preview_field: function() {


		// note we assume weights now exist
		
		// first resample the input field
		// will eventually use createField and setType to do this
		this.resampledInputFieldName = "resampled_" + this.inputFieldName;
		this.commandData.executeCommand("gfx def field " + this.resampledInputFieldName +
															 " image_resample field " + this.inputFieldName + " sizes 128 128;");
		this.resampledInputField = this.commandData.findFieldByName(this.resampledInputFieldName);
		
 		// Set up previewOutputField
		this.previewOutputFieldName = "preview_" + this.outputFieldName;
		this.previewOutputField = this.commandData.createField(this.previewOutputFieldName);

		// set the type with the current options
		this.previewOutputField.setTypeThresholdImageFilter(this.resampledInputField,this.mode,this.outsideValue,this.belowValue,this.aboveValue);

	},

	// should this return the name of the material?  change this to update_preview_field
  preview_filter: function() {

		this.previewOutputField.setTypeThresholdImageFilter(this.resampledInputField,this.mode,this.outsideValue,this.belowValue,this.aboveValue);		

  },


	apply_filter: function() {
	
		this.outputField.setTypeThresholdImageFilter(this.inputField,this.mode,this.outsideValue,this.belowValue,this.aboveValue);
		this.callback.run();

	},

	create_dialog: function() {

		var parent = this.parent;
		var params={generalThresholdFilter:this};

		this.setup_preview_field();

		try {

			dialogReference = parent.openDialog("chrome://zinc/content/filters/generalThresholdFilterDialog.xul",
																					"general-threshold-filter-dialog",
																					"chrome",params);
									
			// store reference to dialog and associated field name
			parent.generalThresholdFilterDialogReference = dialogReference;
			parent.generalThresholdFilterDialogReferenceOutputField = this.outputFieldName;

		}catch(ex) {
			parent.alert("Error creating dialog to edit general threshold filter: " + ex);
		}

	},


  open_dialog: function(gElementName, sceneViewer) {

		this.gElementName = gElementName;
		this.sceneViewer = sceneViewer;

		// parent window
		var parent = this.parent;
		parent.alert("gElementName is " + gElementName);
		var dialogReference = parent.generalThresholdFilterDialogReference;
		
		
		// if no reference to an open dialog already exists create it
		if (dialogReference == null || dialogReference.closed) {
			
			// no dialog so create one
			this.create_dialog();
			
		} else {
			
			// dialog exists, check if it is relevant to this field.				
			var oldOutputFieldName = parent.generalThresholdFilterDialogReferenceOutputField;
			
			if (oldOutputFieldName == this.outputFieldName) {
				
				// correct dialog so bring it to the front
				dialogReference.focus();
				
			} else {
				
				// not correct so close dialog and open a new one
				dialogReference.close();
				this.create_dialog();
				
			}
		}
			
  },

    
  // the QueryInterface function throws an exception
  // if the cmguiIGeneralThresholdFilter interface is not implemented
  // otherwise it returns a reference to the GeneralThresholdFilter object
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(cmguiIGeneralThresholdFilter) &&    
        !aIID.equals(nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID. Eg:

myGeneralThresholdFilter = Components.classes["@bioeng.auckland.ac.nz/helloworld;1"].
                          createInstance(Components.interfaces.cmguiIGeneralThresholdFilter);

***********************************************************/
var GeneralThresholdFilterFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new GeneralThresholdFilter()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var GeneralThresholdFilterModule = {
  _firstTime: true,
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, 
        CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);        
  },
  
  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return GeneralThresholdFilterFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
function NSGetModule(aCompMgr, aFileSpec) { return GeneralThresholdFilterModule; }
